@@ -31,7 +31,6 @@
 #include <QSizePolicy>
 
 #include <KAction>
-#include <KPushButton>
 #include <KGlobal>
 #include <KLocale>
 #include <KStandardAction>
@@ -88,7 +87,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         buttonY=metric1.height()*1.3;
         buttonX=metric1.width(QLatin1String( "00" ));
         mButtonDigit[0]->setText( QString::number(0) );
-        connect( mButtonDigit[0], SIGNAL( clicked() ), this, SLOT( slotDigitClicked() ) );
+        connect( mButtonDigit[0], SIGNAL(clicked()), this, SLOT(slotDigitClicked()) );
         mButtonDigit[0]->setMinimumSize(buttonX,buttonY);
         mButtonDigit[0]->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
         mButtonDigit[0]->setVisible( true );
@@ -99,7 +98,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
             int column = ((i - 1) % 3);
             mButtonDigit[i] = new Plasma::PushButton(m_widget);
             mButtonDigit[i]->setText( QString::number(i) );
-            connect( mButtonDigit[i], SIGNAL( clicked() ), this, SLOT( slotDigitClicked() ) );
+            connect( mButtonDigit[i], SIGNAL(clicked()), this, SLOT(slotDigitClicked()) );
             mButtonDigit[i]->setVisible( true );
             mButtonDigit[i]->setMinimumSize(buttonX,buttonY);
             mButtonDigit[i]->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
@@ -109,7 +108,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonDecimal = new Plasma::PushButton(m_widget);
         mButtonDecimal->setText( KGlobal::locale()->decimalSymbol() );
         m_layout->addItem( mButtonDecimal, 5, 2 );
-        connect( mButtonDecimal, SIGNAL( clicked() ), this, SLOT( slotDecimalClicked() ) );
+        connect( mButtonDecimal, SIGNAL(clicked()), this, SLOT(slotDecimalClicked()) );
         mButtonDecimal->setVisible( true );
         mButtonDecimal->setMinimumSize(buttonX,buttonY);
         mButtonDecimal->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
@@ -120,7 +119,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonEquals->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
         m_layout->addItem( mButtonEquals, 4, 3, 2,1 );
 
-        connect( mButtonEquals, SIGNAL( clicked() ), this, SLOT( slotEqualsClicked() ) );
+        connect( mButtonEquals, SIGNAL(clicked()), this, SLOT(slotEqualsClicked()) );
         mButtonEquals->setVisible( true );
 
         mButtonAdd = new Plasma::PushButton(m_widget);
@@ -129,7 +128,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonAdd->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
 
         m_layout->addItem( mButtonAdd, 3, 3 );
-        connect( mButtonAdd, SIGNAL( clicked() ), this, SLOT( slotAddClicked() ) );
+        connect( mButtonAdd, SIGNAL(clicked()), this, SLOT(slotAddClicked()) );
         mButtonAdd->setVisible( true );
 
         mButtonSubtract = new Plasma::PushButton(m_widget);
@@ -138,7 +137,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonSubtract->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
         m_layout->addItem( mButtonSubtract, 2, 3 );
 
-        connect( mButtonSubtract, SIGNAL( clicked() ), this, SLOT( slotSubtractClicked() ) );
+        connect( mButtonSubtract, SIGNAL(clicked()), this, SLOT(slotSubtractClicked()) );
         mButtonSubtract->setVisible( true );
 
         mButtonMultiply = new Plasma::PushButton(m_widget);
@@ -147,7 +146,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonMultiply->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
         m_layout->addItem( mButtonMultiply, 1, 2 );
 
-        connect( mButtonMultiply, SIGNAL( clicked() ), this, SLOT( slotMultiplyClicked() ) );
+        connect( mButtonMultiply, SIGNAL(clicked()), this, SLOT(slotMultiplyClicked()) );
         mButtonMultiply->setVisible( true );
 
         mButtonDivide = new Plasma::PushButton(m_widget);
@@ -157,7 +156,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
 
         m_layout->addItem( mButtonDivide, 1, 1 );
 
-        connect( mButtonDivide, SIGNAL( clicked() ), this, SLOT( slotDivideClicked() ) );
+        connect( mButtonDivide, SIGNAL(clicked()), this, SLOT(slotDivideClicked()) );
         mButtonDivide->setVisible( true );
 
         mButtonClear = new Plasma::PushButton(m_widget);
@@ -166,7 +165,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonClear->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
 
         m_layout->addItem( mButtonClear, 1, 0 );
-        connect( mButtonClear, SIGNAL( clicked() ), this, SLOT( slotClearClicked() ) );
+        connect( mButtonClear, SIGNAL(clicked()), this, SLOT(slotClearClicked()) );
         mButtonClear->setVisible( true );
 
         mButtonAllClear = new Plasma::PushButton(m_widget);
@@ -175,7 +174,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         mButtonAllClear->setSizePolicy(QSizePolicy(QSizePolicy::Ignored,QSizePolicy::Ignored));
 
         m_layout->addItem( mButtonAllClear, 1, 3);
-        connect( mButtonAllClear, SIGNAL( clicked() ), this, SLOT( slotAllClearClicked() ) );
+        connect( mButtonAllClear, SIGNAL(clicked()), this, SLOT(slotAllClearClicked()) );
         mButtonAllClear->setVisible( true );
 
         // the following three lines are a complete pain and only partially work.
@@ -204,105 +203,105 @@ void CalculatorApplet::keyPressEvent ( QKeyEvent * event )
     case Qt::Key_Return:
     case Qt::Key_Enter:
     {
-        mButtonEquals->nativeWidget()->animateClick();
+        mButtonEquals->click();
         event->accept();
         break;
     }
     case Qt::Key_Plus:
     {
-        mButtonAdd->nativeWidget()->animateClick();
+        mButtonAdd->click();
         event->accept();
         break;
     }
     case Qt::Key_Minus:
     {
-        mButtonSubtract->nativeWidget()->animateClick();
+        mButtonSubtract->click();
         event->accept();
         break;
     }
     case Qt::Key_Asterisk:
     case Qt::Key_multiply:
     {
-        mButtonMultiply->nativeWidget()->animateClick();
+        mButtonMultiply->click();
         event->accept();
         break;
     }
     case Qt::Key_Slash:
     {
-        mButtonDivide->nativeWidget()->animateClick();
+        mButtonDivide->click();
         event->accept();
         break;
     }
     case Qt::Key_1:
     {
-        mButtonDigit[1]->nativeWidget()->animateClick();
+        mButtonDigit[1]->click();
         event->accept();
         break;
     }
     case Qt::Key_2:
     {
-        mButtonDigit[2]->nativeWidget()->animateClick();
+        mButtonDigit[2]->click();
         event->accept();
         break;
     }
     case Qt::Key_3:
     {
-        mButtonDigit[3]->nativeWidget()->animateClick();
+        mButtonDigit[3]->click();
         event->accept();
         break;
     }
     case Qt::Key_4:
     {
-        mButtonDigit[4]->nativeWidget()->animateClick();
+        mButtonDigit[4]->click();
         event->accept();
         break;
     }
     case Qt::Key_5:
     {
-        mButtonDigit[5]->nativeWidget()->animateClick();
+        mButtonDigit[5]->click();
         event->accept();
         break;
     }
     case Qt::Key_6:
     {
-        mButtonDigit[6]->nativeWidget()->animateClick();
+        mButtonDigit[6]->click();
         event->accept();
         break;
     }
     case Qt::Key_7:
     {
-        mButtonDigit[7]->nativeWidget()->animateClick();
+        mButtonDigit[7]->click();
         event->accept();
         break;
     }
     case Qt::Key_8:
     {
-        mButtonDigit[8]->nativeWidget()->animateClick();
+        mButtonDigit[8]->click();
         event->accept();
         break;
     }
     case Qt::Key_9:
     {
-        mButtonDigit[9]->nativeWidget()->animateClick();
+        mButtonDigit[9]->click();
         event->accept();
         break;
     }
     case Qt::Key_0:
     {
-        mButtonDigit[0]->nativeWidget()->animateClick();
+        mButtonDigit[0]->click();
         event->accept();
         break;
     }
     case Qt::Key_Comma:
     case Qt::Key_Period:
     {
-        mButtonDecimal->nativeWidget()->animateClick();
+        mButtonDecimal->click();
         event->accept();
         break;
     }
     case Qt::Key_Escape:
     {
-        mButtonClear->nativeWidget()->animateClick();
+        mButtonClear->click();
         event->accept();
         break;
     }
@@ -327,17 +326,15 @@ void CalculatorApplet::slotDigitClicked()
         waitingForDigit = false;
     }
 
-    inputText += QString::number(newDigit);
-
     if (!inputText.contains(KGlobal::locale()->decimalSymbol())) {
-      //If there is no decimal, then we need to reformat the number
-      double currentValue = KGlobal::locale()->readNumber(inputText);
-      QString localizedString = KGlobal::locale()->formatNumber(currentValue, 0);
-      mOutputDisplay->setText(localizedString);
+        //If there is no decimal, then we need to reformat the number
+        inputText = KGlobal::locale()->formatNumber(KGlobal::locale()->readNumber(inputText) * 10 + newDigit, 0);
     } else {
-      //Once we have a decimal, then all we have to do is append the number
-      mOutputDisplay->setText(mOutputDisplay->text()+QString::number(newDigit));
+        //Once we have a decimal, then all we have to do is append the number
+        inputText.append(QString::number(newDigit));
     }
+
+    mOutputDisplay->setText(inputText);
 }
 
 /*
